import { Request, Response } from "express";
import logger from "../logger/logger";
import {
  isAccountNameValid,
  isPasswordValid,
  isUserIdValid,
  isUsernameValid,
} from "../validator/validator";
import {
  decodeBase64,
  getUserSession,
  hashPassword,
  newUserId,
} from "../utils/utils";
import User from "../models/user";
import { Roles } from "../utils/constants";

export async function AddEmployee(req: Request, res: Response) {
  try {
    let { storeId, username, accountName, password } = req.body;

    // validate request

    if (!storeId || !username || !accountName || !password) {
      return res.status(400).send({ err: "invalid request" });
    }

    // verify if requester is a store master

    const requesterSession = await getUserSession(req);

    if (!requesterSession) {
      logger.debug("Requester session not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    const requester = await User.findOne({
      where: {
        user_id: requesterSession.user_id,
      },
    });

    if (!requester) {
      logger.debug("Requester not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    if (requester.role !== Roles.Master) {
      logger.debug("Requester is not a store master");
      return res.status(401).send({ err: "unauthorized" });
    }

    // validate username and account name

    accountName = accountName.toLowerCase();

    if (
      !isUsernameValid(username) ||
      !(await isAccountNameValid(accountName))
    ) {
      return res.status(400).send({ err: "invalid request" });
    }

    // check if user already exists

    const existingUser = await User.findOne({
      where: {
        account_name: accountName,
      },
    });

    if (existingUser) {
      logger.debug(
        "User already exists with this accountName: %s",
        accountName
      );
      return res.status(400).send({ err: "invalid request" });
    }

    // decode password

    const decodedPassword = decodeBase64(password);

    if (!isPasswordValid(decodedPassword)) {
      logger.debug("Password is not valid");
      return res.status(400).send({ err: "invalid request" });
    }

    // hash password

    const hashedPassword = await hashPassword(decodedPassword);

    // create user

    await User.create({
      user_id: newUserId(),
      store_id: storeId,
      role: Roles.Worker,
      account_name: accountName,
      username: username,
      password: hashedPassword,
    })
      .then(() => {
        res.status(200).send({ msg: "success" });
      })
      .catch((err) => {
        logger.error(err);
        res.status(500).send({ err: "invalid request" });
      });
  } catch (error) {
    logger.error(error);
    res.status(500).send({ err: "invalid request" });
  }
}

export async function GetEmployees(req: Request, res: Response) {
  try {
    let { storeId } = req.params;

    const requesterSession = await getUserSession(req);

    if (!requesterSession) {
      logger.debug("Requester session not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    // find all employees of the requester and select only the username and account name

    const employees = await User.findAll({
      where: {
        store_id: storeId,
      },
      attributes: ["user_id", "username", "account_name"],
    });

    // filter out the requester from the employees

    const filteredEmployees = employees.filter(
      (employee) => employee.user_id !== requesterSession.user_id
    );

    res.status(200).send({ employees: filteredEmployees });
  } catch (error) {
    logger.error(error);
    res.status(500).send({ err: "invalid request" });
  }
}

export async function UpdateEmployee(req: Request, res: Response) {
  try {
    let { userId, username, accountName } = req.body;

    // validate request
    // check if username or account name is provided

    if (!userId || (!username && !accountName)) {
      return res.status(400).send({ err: "invalid request" });
    }

    // validate username and account name

    if (!isUserIdValid(userId)) {
      return res.status(400).send({ err: "invalid request" });
    }

    let update = {};

    if (accountName) {
      accountName = accountName.toLowerCase();

      if (!(await isAccountNameValid(accountName))) {
        res.status(400).send({ err: "invalid request" });
        return;
      }

      update = {
        ...update,
        account_name: accountName,
      };
    }

    if (username) {
      if (!isUsernameValid(username)) {
        res.status(400).send({ err: "invalid request" });
        return;
      }

      update = {
        ...update,
        username: username,
      };
    }

    // verify if requester is a store master

    const requesterSession = await getUserSession(req);

    if (!requesterSession) {
      logger.debug("Requester session not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    const requester = await User.findOne({
      where: {
        user_id: requesterSession.user_id,
      },
    });

    if (!requester) {
      logger.debug("Requester not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    if (requester.role !== Roles.Master) {
      logger.debug("Requester is not a store master");
      return res.status(401).send({ err: "unauthorized" });
    }

    // check if user exists

    const existingUser = await User.findOne({
      where: {
        user_id: userId,
      },
    });

    if (!existingUser) {
      logger.debug("User not found");
      return res.status(400).send({ err: "invalid request" });
    }

    // update user

    await User.update(update, {
      where: {
        user_id: userId,
      },
    })
      .then(() => {
        res.status(200).send({ msg: "success" });
      })
      .catch((err) => {
        logger.error(err);
        res.status(500).send({ err: "invalid request" });
      });
  } catch (error) {
    logger.error(error);
    res.status(500).send({ err: "invalid request" });
  }
}

export async function DeleteEmployee(req: Request, res: Response) {
  try {
    let { userId } = req.body;

    // validate request

    if (!userId) {
      return res.status(400).send({ err: "invalid request" });
    }

    // verify if requester is a store master

    const requesterSession = await getUserSession(req);

    if (!requesterSession) {
      logger.debug("Requester session not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    const requester = await User.findOne({
      where: {
        user_id: requesterSession.user_id,
      },
    });

    if (!requester) {
      logger.debug("Requester not found");
      return res.status(401).send({ err: "unauthorized" });
    }

    if (requester.role !== Roles.Master) {
      logger.debug("Requester is not a store master");
      return res.status(401).send({ err: "unauthorized" });
    }

    // check if user exists

    const existingUser = await User.findOne({
      where: {
        user_id: userId,
      },
    });

    if (!existingUser) {
      logger.debug("User not found");
      return res.status(400).send({ err: "invalid request" });
    }

    // delete user

    await User.destroy({
      where: {
        user_id: userId,
      },
    })
      .then(() => {
        res.status(200).send({ msg: "success" });
      })
      .catch((err) => {
        logger.error(err);
        res.status(500).send({ err: "invalid request" });
      });
  } catch (error) {
    logger.error(error);
    res.status(500).send({ err: "invalid request" });
  }
}
